@@ -21,6 +21,20 @@ if enable_logging:
 else:
     fd = None
 
+app_version = '0.5.4'
+about_name = 'OpenVPN Applet'
+about_text = 'Applet for controlling OpenVPN connections'
+about_authors = 'Mikko Vartiainen'
+about_website = 'http://openvpn-applet.garage.maemo.org/'
+about_bugtracker = 'http://bugs.maemo.org/enter_bug.cgi?product=openvpn'
+about_donate = 'https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=XB5F5CMCPY9SE&lc=FI&item_name=Openvpn%20Applet&currency_code=EUR&bn=PP%2dDonationsBF%3abtn_donateCC_LG%2egif%3aNonHosted'
+donate_wishlist_url = ''
+donate_device_url = ''
+
+
+import sys
+sys.path.insert(0, '/usr/lib/openvpn-applet')
+
 import glob
 import os
 import string
@@ -33,7 +47,7 @@ from sets import Set
 import fileinput
 import re
 from UserDict import UserDict
-
+import socket
 import time
 
 def copy_file(file):
@@ -80,9 +94,10 @@ class vpn():
         self.conf_dir = conf_dir
         self.stopcmd = '/etc/init.d/openvpn'
         self.startcmd = '/usr/sbin/openvpn'
-        self.log_file = '/tmp/'+self.name+'.log'
+        self.socket_name = '/tmp/'+self.name+'.socket'
+        self.socket = None
         parameters = self.__read_conf()
-        print parameters
+        log(parameters)
         if parameters["auth-user-pass"] == True:
             self.user_auth = True
         else:
@@ -92,7 +107,66 @@ class vpn():
             self.pass_auth = True      
         else:
             self.pass_auth = False
-        
+
+    def __socket_connect(self):
+        if self.socket == None:
+            self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+            try:
+                self.socket.connect(self.socket_name)
+            except:
+                log("socket.connect() failed")
+                self.socket = None
+                return False
+            data, address = self.socket.recvfrom(256)
+            log("Connect: ", address, data)
+            return True
+        return False
+
+    def __socket_disconnect(self):
+        if self.socket != None:
+            self.socket.close()
+            self.socket = None
+
+    def __admin_command(self, command, get_response = True):
+        data = ''
+        if self.socket == None: 
+            if self.__socket_connect() == False:
+                return data
+        bytes_sent = self.socket.send(command)
+        log("Command: "+command,bytes_sent)
+        if (get_response):
+            while True:
+                chunk = self.socket.recv(256)
+                data = data+chunk
+                if data[:5] == "ERROR": break
+                if data[-5:] == "END\r\n": break
+                if data[:7] == "SUCCESS": break
+                if chunk == '': break
+            log("Response: "+data[-100:])
+            
+        self.__socket_disconnect()
+        return data
+
+    def get_log(self):
+        data = self.__admin_command("log all\n")
+        while True:
+            mindex = data.find("MANAGEMENT")
+            if mindex == -1: break
+            end = data.find("\n",mindex)
+            if end == -1: end = len(data)
+            start = data.rfind("\n",0,mindex)
+            if start >= mindex or start == -1: start = 0
+            data = data[:start+1]+data[end+1:]
+    
+        return data[:-5]
+
+    def get_statistics(self):
+        data = self.__admin_command("status\n")
+        return data[:-5]
+
+    def get_state(self):
+        data = self.__admin_command("state all\n")
+        return data[:-5]
         
     def get_status(self):
         pid = self.get_pid()
@@ -114,10 +188,10 @@ class vpn():
         return int(pid)
         
     def start(self,user=None,passwd=None,enable_log=False):
-        command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file]
+        command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file,"--management",self.socket_name,"unix"]
         if enable_log:
-            command.extend(['--log',self.log_file,'--verb','3'])
-        print command
+            command.extend(['--verb','6'])
+        log(command)
         process = Popen(command, shell=False, stdin=PIPE, stdout=None, cwd=self.conf_dir)
         input = process.stdin
               
@@ -128,13 +202,12 @@ class vpn():
             print >>input,passwd
 
         input.close()
-        
+
         return process.returncode
         
     def stop(self):
-        print "stop"
-        status = call(['sudo', self.stopcmd, 'stop', self.name])
-        return status
+        log("stop")
+        self.__admin_command("signal SIGHUP\n")
 
     def get_conf_file(self):
         return self.conf_file        
@@ -148,9 +221,6 @@ class vpn():
     def pass_required(self):
         return self.pass_auth
 
-    def get_log_file(self):
-        return self.log_file
-
     def __read_conf(self):
         keywords = ["askpass","auth-user-pass"]
         parameters = dict.fromkeys(keywords,False)
@@ -220,9 +290,6 @@ class vpn_control():
         else:
             return -1
 
-    def get_log_file(self,index):
-        return self.vpn_list[index].get_log_file()
-
     def user_required(self,index):
         return self.vpn_list[index].user_required()
         
@@ -385,12 +452,20 @@ class SelectConnectionDialog(gtk.Dialog):
         pannable_area = hildon.PannableArea()
 
         for vpn_number,v in enumerate(vpn_manager.vpn_list):
+            hbox = gtk.HBox()
             button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
             if v.get_status():
+                stats_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+                stats_button.set_text("Status",'')
+                stats_button.connect("clicked", self.stats_button_clicked_event, vpn_number)
+                hbox.pack_start(stats_button)
+                button.set_style(hildon.BUTTON_STYLE_PICKER)
                 button.set_text("Disconnect from",v.get_name())
             else:
+                button.set_style(hildon.BUTTON_STYLE_NORMAL)
                 button.set_text("Connect to",v.get_name())
-            vbox.pack_start(button)
+            hbox.pack_start(button)
+            vbox.pack_start(hbox)
             button.connect("clicked", self.button_clicked_event, vpn_number)
 
         button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
@@ -405,11 +480,15 @@ class SelectConnectionDialog(gtk.Dialog):
         self.set_size_request(800,vbox.size_request()[1]+10)
         #self.show_all()
 
-
     def button_clicked_event(self, widget, vpn_number):
         self.response(vpn_number)
         self.destroy()
 
+    def stats_button_clicked_event(self, widget, vpn_number):
+        dlg = OpenvpnAppletTestConnection(parent=self, title=self.vpn_manager.get_vpn_name(vpn_number), vpn_conn=self.vpn_manager.vpn_list[vpn_number])            
+        dlg.run()        
+        dlg.destroy()
+
 class OpenvpnAppletSettings(gtk.Dialog):
     def __init__(self,title="OpenVPN connections", parent=None, flags=0, buttons=None, vpn_manager=None, applet=None):
         gtk.Dialog.__init__(self,title, parent, flags, buttons)
@@ -427,11 +506,15 @@ class OpenvpnAppletSettings(gtk.Dialog):
         self.test_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.test_button.set_label("Test")
         self.test_button.connect("clicked", self.test_clicked_event)
+        self.about_button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.about_button.set_label("About")
+        self.about_button.connect("clicked", self.about_clicked_event)
 
         self.selection.set_size_request(-1, 155)
         self.action_area.pack_start(self.new_button)
         self.action_area.pack_start(self.delete_button)
         self.action_area.pack_start(self.test_button)
+        self.action_area.pack_start(self.about_button)
         self.add_button("Close", gtk.RESPONSE_CLOSE)
         self.action_area.show_all()
         self.vbox.show_all()
@@ -509,7 +592,7 @@ class OpenvpnAppletSettings(gtk.Dialog):
     def test_clicked_event(self,widget,data=None):
         vpn_number = self.selection.get_active(0)
         if vpn_number >= 0:
-            log("Item selected")
+            log("Item selected", vpn_number)
             vpn = self.vpn_manager.vpn_list[vpn_number]
             if vpn.user_required() and vpn.pass_required():
                 (user,passwd,cont) = user_pass_dlg(True)
@@ -520,25 +603,36 @@ class OpenvpnAppletSettings(gtk.Dialog):
                 passwd = None
                 cont = True
             if cont:
-                fd = open(vpn.get_log_file(),'w+')
-                fd.close()
-                if vpn.get_status:
+                if vpn.get_status():
                     vpn.stop()
                 vpn.start(user,passwd,True)
 
-            dlg = OpenvpnAppletTestConnection(parent=self,log_file=vpn.get_log_file())            
+            dlg = OpenvpnAppletTestConnection(parent=self, vpn_conn=vpn)            
             dlg.run()        
             dlg.destroy()
             vpn.stop()
 
+    def about_clicked_event(self,widget,data=None):
+        from about import HeAboutDialog
+
+        HeAboutDialog.present(self,
+                about_name,
+                'openvpn-applet',
+                app_version,
+                about_text,
+                '(c) 2008-2010 ' + about_authors,
+                about_website,
+                about_bugtracker,
+                about_donate)
+
 
 class OpenvpnAppletTestConnection(gtk.Dialog):
-    def __init__(self,title="Test Connection", parent=None, flags=0, buttons=None,log_file=None):
+    def __init__(self,title="Test Connection", parent=None, flags=0, buttons=None,vpn_conn=None):
         gtk.Dialog.__init__(self,title, parent, flags, buttons)
-        
-        self.fd = open(log_file,"r+")
-        log(self.fd)
+        self.vpn_conn = vpn_conn        
+
         self.timer = None
+        self.update_func = self.vpn_conn.get_state
         
         self.textbuffer = gtk.TextBuffer()
         self.textview = gtk.TextView(self.textbuffer) 
@@ -553,27 +647,56 @@ class OpenvpnAppletTestConnection(gtk.Dialog):
         self.vbox.pack_start(scrolledwindow)
                 
         save_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-        save_button.set_text("Save Log","")
+        save_button.set_text("Save","")
         save_button.connect("clicked", self.save_button_clicked)
         
-        #self.action_area.pack_start(save_button)
+        log_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        log_button.set_text("View Log","")
+        log_button.connect("clicked", self.log_button_clicked)
+
+        status_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        status_button.set_text("Statistics","")
+        status_button.connect("clicked", self.status_button_clicked)
+
+        state_button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        state_button.set_text("Status","")
+        state_button.connect("clicked", self.state_button_clicked)
+
+        self.action_area.pack_start(log_button)
+        self.action_area.pack_start(status_button)
+        self.action_area.pack_start(state_button)
+        self.action_area.pack_start(save_button)
         self.add_button(gtk.STOCK_CLOSE,gtk.RESPONSE_CLOSE)
         
         self.update_text()
         self.show_all()
+        self.timer = gobject.timeout_add(5000, self.update_text)
     
     def update_text(self):
         log("update_text")
         if self.timer == 'stop':
             return False
-        text = self.fd.read()
-        #TextIter.set_line forward_to_end
-        self.textbuffer.insert(self.textbuffer.get_end_iter(), text)
-        #self.textbuffer.insert_at_cursor(text)
-        self.timer = gobject.timeout_add(1000, self.update_text)
+        if self.vpn_conn != None:
+            text = self.update_func()
+            self.textbuffer.set_text(text)
+            self.textview.scroll_to_iter(self.textbuffer.get_end_iter(),0.0,False,0,0)
+        return True
     
+    def log_button_clicked(self, widget, data=None):
+        self.update_func = self.vpn_conn.get_log
+        self.update_text()
+
+    def status_button_clicked(self, widget, data=None):
+        self.update_func = self.vpn_conn.get_statistics
+        self.update_text()
+
+    def state_button_clicked(self, widget, data=None):
+        self.update_func = self.vpn_conn.get_state
+        self.update_text()
+
     def save_button_clicked(self, widget, data=None):
         dlg = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_SAVE,  hildon.FileSystemModel())
+        dlg.set_current_folder("/home/user/MyDocs")
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             filename = dlg.get_filename()
@@ -586,7 +709,6 @@ class OpenvpnAppletTestConnection(gtk.Dialog):
     
     def destroy(self):
         self.timer = 'stop'
-        self.fd.close()
         super(OpenvpnAppletTestConnection,self).destroy()
     
 class OpenvpnAppletNewConnection(gtk.Dialog):
